package amc.rtfms.givemecoffee;
 
import edu.cmu.sphinx.frontend.util.Microphone;
import edu.cmu.sphinx.recognizer.Recognizer;
import edu.cmu.sphinx.result.Result;
import edu.cmu.sphinx.util.props.ConfigurationManager;

import com.sun.speech.freetts.Voice;
import com.sun.speech.freetts.VoiceManager;

import gnu.io.CommPort;
import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;

import java.io.*;
import java.util.*;

public class GiveMeCoffee {

	// Sphinx code ---------------------------------------------------
	// Parts of code from http://cmusphinx.sourceforge.net/
	private static ConfigurationManager getSphinxConfigurationManager( String[] args ) {
        return new ConfigurationManager( GiveMeCoffee.class.getResource( "givemecoffee.config.xml" ) );
	}
	
	private static Microphone getSphinxMicrophone( ConfigurationManager cm, Recognizer recognizer ) {
		Microphone microphone = (Microphone)cm.lookup( "microphone" );
        if( !microphone.startRecording() ) {
            System.out.println( "Cannot start microphone." );
            recognizer.deallocate();
            System.exit(1);
        }
        return microphone;
	}
	
	private static void printHelp( String[] args ) {
		System.out.println( "GiveMeCoffee - for more details see RTFMs vlog episode #3" );
		listAllPorts();
		listAllVoices();
		System.out.println();
		if( args.length > 1 ) {
			System.out.println( "Say: (Computer give me coffee) | (Thank you)" );
			System.out.println( "Keep speaking. Press Ctrl-C to quit.\n" );
		} else {
			System.exit( 3 );
		}
		System.out.println( "\nPlease wait, the system is not ready yet ..." );
	}
	
	// Serial code -----------------------------------------------------------
	// See http://en.wikibooks.org/wiki/Serial_Programming/Serial_Java for details

	private static void listAllPorts() {
        System.out.println("\nAvailable ports:");    
		Enumeration<CommPortIdentifier> portIdentifiers = CommPortIdentifier.getPortIdentifiers();
		while( portIdentifiers.hasMoreElements() )
		{
		    CommPortIdentifier port = portIdentifiers.nextElement();
		    System.out.println( "    " + port.getName() );
		}
		
    }
	
	private static OutputStreamWriter getSerialPortStream( String[] args ) {
    	OutputStreamWriter out = null;
    	try {
    		System.out.println( "Using serial port: " + args[0] );
	    	CommPortIdentifier portIdentifier = CommPortIdentifier.getPortIdentifier( args[0] );
	    	if( portIdentifier.isCurrentlyOwned() ) {
	            System.out.println("Error: Port is currently in use");
	        } else {
	            CommPort commPort = portIdentifier.open("GiveMeCoffee",2000);
	            if ( commPort instanceof SerialPort ) {
	                SerialPort serialPort = (SerialPort) commPort;
	                serialPort.setSerialPortParams(115200,SerialPort.DATABITS_8,SerialPort.STOPBITS_1,SerialPort.PARITY_NONE);
	                //InputStream in = serialPort.getInputStream();
	                out = new OutputStreamWriter(serialPort.getOutputStream());
	            } else {
	                System.out.println("Error: Only serial ports are handled by this example.");
	            }
	        }
    	} catch( Exception e ) {
    		System.out.println( "Ooops, " + e );
    		System.exit( 2 );
    	}
    	return out;
    }
    
    // TTS code ---------------------------------------------
    
    private static void listAllVoices() {
        System.out.println("\nAvailable voices:");        
        VoiceManager voiceManager = VoiceManager.getInstance();
        Voice[] voices = voiceManager.getVoices();
        for (int i = 0; i < voices.length; i++) {
            System.out.println("    " + voices[i].getName()
                               + " (" + voices[i].getDomain() + " domain)");
        }
    }
    
    private static Voice getVoice( String[] args ) {
    	System.out.println( "Using voice: " + args[1] );
        VoiceManager voiceManager = VoiceManager.getInstance();
        Voice voice = voiceManager.getVoice( args[1] );
        if (voice == null) {
            System.err.println( "Cannot find a voice named '" + args[1] + "'.  Please specify a different voice.");
            System.exit(1);
        }
        voice.allocate();
        return voice;
    }
    
    private static void say( Voice voice, Microphone mic, Recognizer recognizer, String text ) {
    	voice.speak( text );
    	mic.clear();
    }
    
    // Main -------------------------------------------------
    
	public static void main( String[] args ) {
        printHelp( args );
        ConfigurationManager cm = getSphinxConfigurationManager( args );
        Recognizer recognizer = (Recognizer) cm.lookup( "recognizer" );
        recognizer.allocate();
        Microphone mic = getSphinxMicrophone( cm, recognizer );
        OutputStreamWriter servo = getSerialPortStream( args );
        Voice voice = getVoice( args );
		System.out.println( "Ok, ready now.\n******************************************\n" );
        
        while( true ) {
            Result result = recognizer.recognize();
            if( result != null ) {
                String resultText = result.getBestFinalResultNoFiller();
                System.out.println( "Token: " + result.getBestFinalToken());
                System.out.println( "BPR: " + result.getBestPronunciationResult());
                System.out.println( "BRNF: " + result.getBestResultNoFiller() );
                if( resultText == "" ) {
                	System.out.println( "I can't understand what you said." );
                	continue;
                }
                System.out.println("You said: '" + resultText + "'");
                try {
	                if( resultText.equals( "computer give me coffee" ) ) {
	                	servo.write( "#0P1150S800\r" );
	                	servo.flush();
	                	say( voice, mic, recognizer, "Here it is, sir." );
	                } else if( resultText.equals( "hey computer thank you" ) ) {
	                	servo.write( "#0P2230S800\r" );               	                 
	                	servo.flush();
	                	say( voice, mic, recognizer, "You are welcome, sir." );
	                }
                } catch( Exception e ) {
                	System.out.println( "Ooooooops, " + e );
                }
            } else {
                System.out.println( "I can't hear you." );
            }
            System.out.println();
        }
	}

}
